@@ -4,6 +4,7 @@
   SPDX-FileCopyrightText: 2001 Matthias Hoelzer-Kluepfel <mhk@kde.org>
   SPDX-FileCopyrightText: 2002-2005 Daniel Molkentin <molkentin@kde.org>
   SPDX-FileCopyrightText: 2003-2005 Cornelius Schumacher <schumacher@kde.org>
+  SPDX-FileCopyrightText: 2021 Alexander Lohnau <alexander.lohnau@gmx.de>
 
   SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -12,7 +13,6 @@
 #include "aboutdialog.h"
 #include "iconsidepane.h"
 #include "prefs.h"
-
 #include "webengine/introductionwebenginepage.h"
 #include "webengine/introductionwebengineview.h"
 
@@ -64,8 +64,6 @@ using namespace Kontact;
 #include <QVBoxLayout>
 #include <QWebEngineUrlScheme>
 
-#include <ksettings/Dispatcher>
-
 #include <GrantleeTheme/GrantleeTheme>
 #include <GrantleeTheme/GrantleeThemeManager>
 
@@ -134,17 +132,9 @@ void MainWindow::initObject()
             qFatal("KSycoca unavailable. Kontact will be unable to find plugins.");
         }
     }
-
-    const QVector<KPluginMetaData> pluginMetaDatas = KPluginLoader::findPlugins(QStringLiteral("kontact5"), [](const KPluginMetaData &data) {
+    mPluginMetaData = KPluginMetaData::findPlugins(QStringLiteral("kontact5"), [](const KPluginMetaData &data) {
         return data.rawData().value(QStringLiteral("X-KDE-KontactPluginVersion")).toInt() == KONTACT_PLUGIN_VERSION;
     });
-    mPluginInfos = KPluginInfo::fromMetaData(pluginMetaDatas);
-
-    KConfigGroup configGroup(Prefs::self()->config(), "Plugins");
-    for (KPluginInfo &pluginInfo : mPluginInfos) {
-        pluginInfo.setConfig(configGroup);
-        pluginInfo.load();
-    }
 
     // prepare the part manager
     mPartManager = new KParts::PartManager(this);
@@ -232,7 +222,7 @@ void MainWindow::initWidgets()
 {
     auto mTopWidget = new QWidget(this);
     auto layout = new QVBoxLayout;
-    layout->setContentsMargins(0, 0, 0, 0);
+    layout->setContentsMargins({});
     layout->setSpacing(0);
     mTopWidget->setLayout(layout);
     setCentralWidget(mTopWidget);
@@ -278,7 +268,7 @@ void MainWindow::initAboutScreen()
 {
     auto introbox = new QWidget(mPartsStack);
     auto introboxHBoxLayout = new QHBoxLayout(introbox);
-    introboxHBoxLayout->setContentsMargins(0, 0, 0, 0);
+    introboxHBoxLayout->setContentsMargins({});
     mPartsStack->addWidget(introbox);
     mPartsStack->setCurrentWidget(introbox);
     mIntroPart = new IntroductionWebEngineView(introbox);
@@ -329,52 +319,54 @@ KontactInterface::Plugin *MainWindow::pluginFromName(const QString &identifier)
 void MainWindow::loadPlugins()
 {
     QList<KontactInterface::Plugin *> plugins;
-    for (const KPluginInfo &pluginInfo : std::as_const(mPluginInfos)) {
-        if (!pluginInfo.isPluginEnabled()) {
+    KConfigGroup configGroup(Prefs::self()->config(), "Plugins");
+    for (const KPluginMetaData &pluginMetaData : std::as_const(mPluginMetaData)) {
+        if (!configGroup.readEntry(pluginMetaData.pluginId() + QLatin1String("Enabled"), pluginMetaData.isEnabledByDefault())) {
             continue;
         }
 
-        const QString pluginPath = pluginInfo.pluginName();
-        KontactInterface::Plugin *plugin = pluginFromName(pluginPath);
+        KontactInterface::Plugin *plugin = pluginFromName(pluginMetaData.pluginId());
         if (plugin) { // already loaded
             plugin->configUpdated();
             continue;
         }
 
-        qCDebug(KONTACT_LOG) << "Loading Plugin:" << pluginInfo.name();
-        KPluginLoader loader(pluginInfo.libraryPath());
-        KPluginFactory *factory = loader.factory();
-        if (!factory) {
-            qWarning() << "Error loading plugin" << pluginInfo.libraryPath() << loader.errorString();
+        qCDebug(KONTACT_LOG) << "Loading Plugin:" << pluginMetaData.name();
+        const auto loadResult = KPluginFactory::instantiatePlugin<KontactInterface::Plugin>(pluginMetaData, this);
+        if (!loadResult) {
+            qCWarning(KONTACT_LOG) << "Error loading plugin" << pluginMetaData.fileName() << loadResult.errorString;
             continue;
         } else {
-            plugin = factory->create<KontactInterface::Plugin>(this);
+            plugin = loadResult.plugin;
             if (!plugin) {
-                qCWarning(KONTACT_LOG) << "Unable to create plugin for" << pluginInfo.libraryPath();
+                qCWarning(KONTACT_LOG) << "Unable to create plugin for" << pluginMetaData.fileName();
                 continue;
             }
         }
+        plugin->setIdentifier(pluginMetaData.fileName());
+        plugin->setTitle(pluginMetaData.name());
+        plugin->setIcon(pluginMetaData.iconName());
 
-        plugin->setIdentifier(pluginInfo.pluginName());
-        plugin->setTitle(pluginInfo.name());
-        plugin->setIcon(pluginInfo.icon());
+        const QString libNameProp = pluginMetaData.value(QStringLiteral("X-KDE-KontactPartLibraryName"));
+        const QString exeNameProp = pluginMetaData.value(QStringLiteral("X-KDE-KontactPartExecutableName"));
 
-        const QVariant libNameProp = pluginInfo.property(QStringLiteral("X-KDE-KontactPartLibraryName"));
-        const QVariant exeNameProp = pluginInfo.property(QStringLiteral("X-KDE-KontactPartExecutableName"));
-        const QVariant loadOnStart = pluginInfo.property(QStringLiteral("X-KDE-KontactPartLoadOnStart"));
-        const QVariant hasPartProp = pluginInfo.property(QStringLiteral("X-KDE-KontactPluginHasPart"));
-
-        if (!loadOnStart.isNull() && loadOnStart.toBool()) {
-            mDelayedPreload.append(plugin);
+        if (pluginMetaData.rawData().contains(QLatin1String("X-KDE-KontactPartLoadOnStart"))) {
+            const bool loadOnStart = pluginMetaData.rawData().value(QStringLiteral("X-KDE-KontactPartLoadOnStart")).toBool();
+            if (loadOnStart) {
+                mDelayedPreload.append(plugin);
+            }
+        }
+        if (pluginMetaData.rawData().contains(QLatin1String("X-KDE-KontactPluginHasPart"))) {
+            const bool hasPartProp = pluginMetaData.rawData().value(QStringLiteral("X-KDE-KontactPluginHasPart")).toBool();
+            plugin->setShowInSideBar(hasPartProp);
+        } else {
+            plugin->setShowInSideBar(true);
         }
 
-        qCDebug(KONTACT_LOG) << "LIBNAMEPART:" << libNameProp.toString();
+        qCDebug(KONTACT_LOG) << "LIBNAMEPART:" << libNameProp;
 
-        plugin->setPartLibraryName(libNameProp.toString().toUtf8());
-        plugin->setExecutableName(exeNameProp.toString());
-        if (hasPartProp.isValid()) {
-            plugin->setShowInSideBar(hasPartProp.toBool());
-        }
+        plugin->setPartLibraryName(libNameProp.toUtf8());
+        plugin->setExecutableName(exeNameProp);
         plugins.append(plugin);
     }
     std::sort(plugins.begin(), plugins.end(), pluginWeightLessThan); // new plugins
@@ -404,16 +396,17 @@ void MainWindow::unloadDisabledPlugins()
 {
     // Only remove the now-disabled plugins.
     // Keep the other ones. loadPlugins() will skip those that are already loaded.
-    for (const KPluginInfo &pluginInfo : std::as_const(mPluginInfos)) {
-        if (!pluginInfo.isPluginEnabled()) {
-            removePlugin(pluginInfo.pluginName());
+    KConfigGroup configGroup(Prefs::self()->config(), "Plugins");
+    for (const KPluginMetaData &pluginMetaData : std::as_const(mPluginMetaData)) {
+        if (!configGroup.readEntry(pluginMetaData.pluginId() + QLatin1String("Enabled"), pluginMetaData.isEnabledByDefault())) {
+            removePlugin(pluginMetaData.pluginId());
         }
     }
 }
 
 void MainWindow::updateShortcuts()
 {
-    for (int i = 0; i < mActionPlugins.count(); ++i) {
+    for (int i = 0, total = mActionPlugins.count(); i < total; ++i) {
         QAction *action = mActionPlugins.at(i);
         const QKeySequence shortcut(QStringLiteral("Ctrl+%1").arg(mActionPlugins.count() - i));
         actionCollection()->setDefaultShortcut(action, shortcut);
@@ -719,30 +712,27 @@ void MainWindow::slotPreferences()
     static Kontact::KontactConfigureDialog *dlg = nullptr;
     if (!dlg) {
         dlg = new Kontact::KontactConfigureDialog(this);
-        dlg->setAllowComponentSelection(true);
-        connect(dlg, QOverload<const QByteArray &>::of(&KSettings::Dialog::configCommitted), this, [this](const QByteArray &componentName) {
+        connect(dlg, &KontactSettingsDialog::configCommitted, this, [this](const QByteArray &componentName) {
             if (componentName == QByteArrayLiteral("kontact")) {
                 MainWindow::updateConfig();
             }
         });
 
-        // do not show settings of components running standalone
-        KPluginInfo::List filteredPlugins = mPluginInfos;
-        PluginList::ConstIterator end(mPlugins.constEnd());
-        for (PluginList::ConstIterator it = mPlugins.constBegin(); it != end; ++it) {
-            if ((*it)->isRunningStandalone()) {
-                KPluginInfo::List::ConstIterator infoIt;
-                KPluginInfo::List::ConstIterator infoEnd(filteredPlugins.constEnd());
-                for (infoIt = filteredPlugins.constBegin(); infoIt != infoEnd; ++infoIt) {
-                    if (infoIt->pluginName() == (*it)->identifier()) {
-                        filteredPlugins.removeAll(*infoIt);
-                        break;
-                    }
-                }
+        // Add the main contact KCM which is not associated with a specific plugin
+        dlg->addModule(KPluginMetaData(QStringLiteral("pim/kcms/kontact/kcm_kontact")));
+
+        auto sortByWeight = [](const KPluginMetaData &m1, const KPluginMetaData &m2) {
+            return m1.rawData().value(QStringLiteral("X-KDE-Weight")).toInt() < m2.rawData().value(QStringLiteral("X-KDE-Weight")).toInt();
+        };
+        std::sort(mPluginMetaData.begin(), mPluginMetaData.end(), sortByWeight);
+        for (const KPluginMetaData &metaData : std::as_const(mPluginMetaData)) {
+            const QString pluginNamespace = metaData.value(QStringLiteral("X-KDE-ConfigModuleNamespace"));
+            if (!pluginNamespace.isEmpty()) {
+                auto plugins = KPluginMetaData::findPlugins(pluginNamespace);
+                std::sort(plugins.begin(), plugins.end(), sortByWeight);
+                dlg->addPluginComponent(metaData, plugins);
             }
         }
-        dlg->addPluginInfos(filteredPlugins);
-        connect(dlg, &Kontact::KontactConfigureDialog::pluginSelectionChanged, this, &MainWindow::pluginsChanged);
     }
 
     dlg->show();
@@ -781,7 +771,7 @@ void MainWindow::showAboutDialog()
 
 void MainWindow::configureShortcuts()
 {
-    KShortcutsDialog dialog(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsAllowed, this);
+    KShortcutsDialog dialog(this);
     dialog.addCollection(actionCollection());
 
     if (mCurrentPlugin && mCurrentPlugin->part()) {
@@ -865,9 +855,10 @@ void MainWindow::saveProperties(KConfigGroup &config)
 
     QStringList activePlugins;
 
-    for (const KPluginInfo &pluginInfo : std::as_const(mPluginInfos)) {
-        if (pluginInfo.isPluginEnabled()) {
-            KontactInterface::Plugin *plugin = pluginFromName(pluginInfo.pluginName());
+    KConfigGroup configGroup(Prefs::self()->config(), "Plugins");
+    for (const KPluginMetaData &pluginMetaData : std::as_const(mPluginMetaData)) {
+        if (!configGroup.readEntry(pluginMetaData.pluginId() + QLatin1String("Enabled"), pluginMetaData.isEnabledByDefault())) {
+            KontactInterface::Plugin *plugin = pluginFromName(pluginMetaData.pluginId());
             if (plugin) {
                 activePlugins.append(plugin->identifier());
                 plugin->saveProperties(config);
